const Joi = require('@hapi/joi');
const error = require('http-errors');

const api = require('.');
const zmqUtils = require('../../../routes/utils_zmq');

const { uid } = require('../../../node_modules/schemas/common');
const { positionObj } = require('../../../node_modules/schemas/win_anchor');

async function getWinAnchorSyncHandler(req, res, next) {
    try {
        const win_anchors = await api.getWinAnchorSync(req.scope.projectTag, req.scope.subcontractorTag, req.params.uid);
        res.json({ win_anchors });
    } catch (e) {
        next(e);
    }
}

async function getAllWinAnchorSyncHandler(req, res, next) {
    try {
        const win_anchors = await api.getWinAnchorSync(req.scope.projectTag, req.scope.subcontractorTag);
        res.json({ win_anchors });
    } catch (e) {
        next(e);
    }
}

async function editWinAnchorSyncHandler(req, res, next) {
    try {
        const out = await api.updateWinAnchorSync(req.params.uid, req.data, req.scope.projectTag, req.scope.subcontractorTag);
        const updateWinAnchor = out[0];
        zmqUtils.notify({
            req,
            object: zmqUtils.OBJECT_TYPE.ZMQ_OBJECT.WIN_ANCHOR,
            action: zmqUtils.OBJECT_TYPE.TYPE_ACTION.UPDATED,
            plist: updateWinAnchor,
        });

        res.json(updateWinAnchor);
    } catch (e) {
        next(e);
    }
}


const numberSchema = Joi.number().min(0).max(31).integer()
    .allow(255)
    .error(error(422, 'sync_slot.slot should be from 0 to 31, or 255'));

const sync_slot = Joi.object({
    addr: Joi.string().min(4).max(4).regex(/^([0-9a-fA-F])+$/)
        .required()
        .error(error(422, 'sync_slot.addr should be hex based string 2 bytes')),
    slot: numberSchema,
    flags: Joi.string().min(2).max(2).regex(/^([0-9a-fA-F])+$/)
        .default('00')
        .error(error(422, 'sync_slot.flags should be hex based string 1 bytes')),
}).allow(null);

const updateSyncWinAnchor = Joi.object({
    uid,
    name: Joi.string(),
    position: positionObj.optional(),
    type: Joi.string(),
    bt_config: Joi.object().allow(null),
    acc_config: Joi.object().allow(null),
    state: Joi.object().allow(null),
    locked: Joi.boolean().optional(),
    sync_slot,
});

exports.getAllWinAnchorSyncHandler = getAllWinAnchorSyncHandler;
exports.getWinAnchorSyncHandler = getWinAnchorSyncHandler;
exports.editWinAnchorSyncHandler = editWinAnchorSyncHandler;
exports.updateSyncWinAnchor = updateSyncWinAnchor;
